/*
 *  Lightwave3D Gif Animation Player
 *     by Arnie Cachelin, NewTek, Inc.
 *  Requires animator.class by Herb Jellinek, Sun Microsystems, Inc.
 *
 */

import java.io.InputStream;
import java.awt.*;
import java.awt.image.ImageProducer;
import java.applet.Applet;
import java.applet.AudioClip;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;

/* This class adapts the Animator class to handle images produced by Lightwave3D
 * and most other animation programs, by adding and replacing methods to handle
 * image filenames other than "T", and numbers of digits of "0" padding as in
 * "base007.gif".  Use the "BASENAME" parameter to specify the beginning of the
 * filename, and "DIGITS" to specify the number of characters in the
 */
public class LWAnimator extends Animator implements Runnable {

    /**
     * Info.
     */
    public String getAppletInfo() {
	return "LWAnimator by  Arnie Cachelin";
    }

    /**
     * Parameter Info
     */
    public String[][] getParameterInfo() {
	String[][] info = {
	    {"imagesource", 	"url", 		"a directory"},
	    {"startup", 	"url", 		"displayed at startup"},
	    {"background", 	"url", 		"displayed as background"},
	    {"startimage", 	"int", 		"start index"},
	    {"endimage", 	"int", 		"end index"},
	    {"pause", 	        "int", 		"milliseconds"},
	    {"pauses", 	        "ints", 	"milliseconds"},
	    {"repeat", 	        "boolean", 	"repeat or not"},
	    {"positions",	"coordinates", 	"path"},
	    {"soundsource",	"url", 		"audio directory"},
	    {"soundtrack",	"url", 		"background music"},
	    {"sounds",		"urls",		"audio samples"},
		{"basename",	"string",   "image file base name"},
		{"digits",	    "int",      "# of digits appended to base name"}
	};
	return info;
    }

    /**
     * Fix frame digits base on start,end image numbers.
     */
    int setFrameDigits(int startImage, int endImage)
    {
        dbg("Frames "+startImage + " to "+endImage+" digits = "+frameDigits);
    	if(frameDigits>0) {
            String s;
            int n;
            s = String.valueOf( (startImage > endImage) ? startImage:endImage);
            n = s.length();
            if(n>frameDigits)
                frameDigits = n;
        }
        dbg("digits = "+frameDigits);
        return frameDigits;
    }


    /**  !! Overriding existing Animator method...
     * Stuff a range of image names into a Vector.
     * @return a Vector of image URLs.
     */
    Vector prepareImageRange(int startImage, int endImage)
    throws MalformedURLException {
	Vector result = new Vector(Math.abs(endImage - startImage) + 1);
	if (startImage > endImage) {
	    for (int i = startImage; i >= endImage; i--) {
	    	if(frameDigits>1)
	    	{
	    		String num = new String("0000"+i);
	    		num = new String(num.substring(num.length()-frameDigits,num.length()));
				result.addElement(new URL(imageSource, baseName+num+".gif"));
				dbg(baseName+num+".gif");
	    	}
	   		else
				result.addElement(new URL(imageSource, baseName+i+".gif"));
	    }
	} else {
	    for (int i = startImage; i <= endImage; i++) {
	    	if(frameDigits>1)
	    	{
	    		String num = new String("0000"+i);
	    		num = new String(num.substring(num.length()-frameDigits,num.length()));
				result.addElement(new URL(imageSource, baseName+num+".gif"));
				dbg(baseName+num+".gif");
	    	}
	   		else
				result.addElement(new URL(imageSource, baseName+i+".gif"));
	    }
	}
	return result;
    }


    /** !! Overriding existing Animator method...
     * Initialize the applet.  Get parameters.
     */
    public void init() {

	try {
	    String param = getParameter("IMAGESOURCE");
	    imageSource = (param == null) ? getDocumentBase() : new URL(getDocumentBase(), param + "/");
	    dbg("IMAGESOURCE = "+param);

	    param = getParameter("PAUSE");
	    globalPause =
		(param != null) ? Integer.parseInt(param) : defaultPause;
	    dbg("PAUSE = "+param);

	    param = getParameter("REPEAT");
	    repeat = (param == null) ? true : (param.equalsIgnoreCase("yes") ||
					       param.equalsIgnoreCase("true"));

	    param = getParameter("BASENAME");
	    dbg("BASENAME = "+param);
	    if (param != null) {
			baseName =  new String(param);
	    }
	    param = getParameter("DIGITS");
	    dbg("DIGITS = "+param);
	    if (param != null) {
			frameDigits =  Integer.parseInt(param);
	    }
	    int startImage = 1;
	    int endImage = 1;
	    param = getParameter("ENDIMAGE");
	    dbg("ENDIMAGE = "+param);
	    if (param != null) {
    		endImage = Integer.parseInt(param);
    		param = getParameter("STARTIMAGE");
    		dbg("STARTIMAGE = "+param);
    		if (param != null) {
    		    startImage = Integer.parseInt(param);
    		}
    		setFrameDigits(startImage,endImage);
    		images = prepareImageRange(startImage, endImage);
	    } else {
       		param = getParameter("STARTIMAGE");
    		dbg("STARTIMAGE = "+param);
    		if (param != null) {
    		    startImage = Integer.parseInt(param);
        		setFrameDigits(startImage,endImage);
    		    images = prepareImageRange(startImage, endImage);
    		} else {
    		    param = getParameter("IMAGES");
    		    if (param == null) {
        			showStatus("No legal IMAGES, STARTIMAGE, or ENDIMAGE "+
        				   "specified.");
        			return;
    		    } else {
        			images = parseImages(param);
    		    }
    		}
    	}

	    param = getParameter("BACKGROUND");
	    dbg("BACKGROUND = "+param);
	    if (param != null) {
		backgroundImageURL = new URL(imageSource, param);
	    }

	    param = getParameter("STARTUP");
	    dbg("STARTUP = "+param);
	    if (param != null) {
		startUpImageURL = new URL(imageSource, param);
	    }

	    param = getParameter("SOUNDSOURCE");
	    soundSource = (param == null) ? imageSource : new URL(getDocumentBase(), param + "/");
	    dbg("SOUNDSOURCE = "+param);

	    param = getParameter("SOUNDS");
	    dbg("SOUNDS = "+param);
	    if (param != null) {
		sounds = parseSounds(param, images);
	    }

	    param = getParameter("PAUSES");
	    dbg("PAUSES = "+param);
	    if (param != null) {
		durations = parseDurations(param, images);
	    }

	    param = getParameter("POSITIONS");
	    dbg("POSITIONS = "+param);
	    if (param != null) {
		positions = parsePositions(param, images);
	    }

	    param = getParameter("SOUNDTRACK");
	    dbg("SOUNDTRACK = "+param);
	    if (param != null) {
			soundtrackURL = new URL(soundSource, param);
		}
	} catch (MalformedURLException e) {
	    showParseError(e);
	} catch (ParseException e) {
	    showParseError(e);
	}

	setFrameNum(0);

   }

	public String baseName = "T"; // For default compatibility behaviour
	public int frameDigits = 0;   // 0 for no "0" padding of file numbers
}


class ParseException extends Exception {
    ParseException(String s) {
	super(s);
    }
}

class ImageNotFoundException extends Exception {
    ImageNotFoundException(ImageProducer source) {
	super(source+"");
    }
}

